/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GeneClust user interface (GCAboutPanel)
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   Kim-Anh Do
 *     Application::  GeneClust
 *
 *============================================================================*/

/*
 * @(#)GCAboutPanel.java    1.0 2001/06/07
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.geneclust;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.net.URL;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.mdanderson.biostat.util.DefaultAboutPanel;
import org.mdanderson.biostat.util.MdaccIcon;


/**
 * GCAboutPanel - Creates an ABOUT panel for this application
 *
 * @version 1.2 2002/01/13
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class GCAboutPanel extends DefaultAboutPanel
    implements GeneClustKeys
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public GCAboutPanel()
    {
        super();
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Gets the preferred size of the component
     *
     * @return  dimension object indicating this component's preferred size
     */
    public Dimension getPreferredSize()
    {
        final int WIDTH = 420;
        final int HEIGHT = 215;

        return new Dimension(WIDTH, HEIGHT);
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Overrides the parent routine to place things differently.
     * This particular application has no 'imageIcon' itself so
     * creation of that element's panel is also skipped.
     */
    protected void addAboutPanelItems()
    {
        JPanel bannerPanel = createBannerPanel();
        JPanel infoPanel   = createInfoPanel();
        JPanel logoPanel   = createLogoPanel();

        add(bannerPanel, BorderLayout.NORTH);
        add(logoPanel, BorderLayout.EAST);
        add(Box.createHorizontalGlue(), BorderLayout.CENTER);
        add(infoPanel, BorderLayout.WEST);
    }

    /**
     * Overrides the parent routine to add a microarray border under
     * the banner name. Nice effect.
     *
     * @return  instance of <code>JPanel</code> class
     */
    protected JPanel createBannerPanel()
    {
        JPanel bannerPanel = super.createBannerPanel();

        // Set text to application-specific values
        setBannerLine(getBundleString(PACKAGE_NAME));

        // Add bottom border image
        {
            final String borderFileName = "microarray_border.jpg";
            URL borderUrl = getResource(borderFileName);
            ImageIcon borderImage = new ImageIcon(borderUrl);

            JLabel label = new JLabel(borderImage);
            if (borderUrl == null)
                label.setText("***Missing image***");
            bannerPanel.add(label);
        }

        return bannerPanel;
    }

    /**
     * Overrides the parent routine to change the labels.
     *
     * @return  instance of <code>JPanel</code> class
     */
    protected JPanel createInfoPanel()
    {
        JPanel infoPanel = super.createInfoPanel();

        // Set application name/version foreground to MDACC color
        appNameVersionLine.setForeground(GCAboutPanel.mystery);

        // Set text to application-specific values
        setAppVersionLine(getBundleString(APPLICATION_NAME),
                          getBundleString(APPLICATION_VERSION));
        setReleaseDateLine(getBundleString(APPLICATION_RELEASE_DATE));
        setCopyrightLine(getBundleString(APPLICATION_COPYRIGHT));
        setLegalitiesLine(getBundleString(APPLICATION_LEGALITIES));

        return infoPanel;
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Creates a panel containing the MDACC logo
     *
     * @return  instance of <code>JPanel</code> class
     */
    private JPanel createLogoPanel()
    {
        JPanel logoPanel = new JPanel();
        logoPanel.setLayout(new BoxLayout(logoPanel, BoxLayout.X_AXIS));

        // Add elements
        {
            final int WIDTH = 108;
            final int HEIGHT = 75;

            JLabel logo = new JLabel(new MdaccIcon());
            logo.setMaximumSize(new Dimension(WIDTH, HEIGHT));
            logoPanel.add(logo);
        }

        return logoPanel;
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return GeneClust.getBundleString(propertyKey);
    }

    /**
     * Finds a resource with a given name. This method returns null if no
     * resource with this name is found. The rules for searching resources
     * associated with a given class are implemented by the defining class
     * loader of the class.
     *
     * @param resourceName  string specifying name of the desired resource
     * @return              url of requested resource
     * @see Class#getResource
     */
    static URL getResource(String resourceName)
    {
        return GeneClust.getResource(resourceName);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static Color mystery;

    static
    {
        {
            final int[] rgbMystery = {  0, 120, 136 };
            mystery = new Color(rgbMystery[0],
                                rgbMystery[1],
                                rgbMystery[2]);
        }
    }
}

