/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for abstract dialog class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)AbstractDialog.java    1.0 2002/03/02
 *
 * Copyright (c) 2002 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.SwingUtilities;


/**
 * AbstractDialog - Creates an abstract dialog class
 *
 * @version 1.0 2002/03/02
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
abstract public class AbstractDialog extends JDialog
    implements Observer
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying title
     *
     * @param title  string specifying dialog title
     */
    public AbstractDialog(String title)
    {
        this((Frame)null, title);
    }

    /**
     * Class constructor specifying owner and title
     *
     * @param owner  parent component
     * @param title  string specifying dialog title
     */
    public AbstractDialog(Frame owner, String title)
    {
        super(owner, title);

        controlArea = createControlArea();
        actionArea = createActionArea();

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(controlArea, BorderLayout.CENTER);
        getContentPane().add(actionArea, BorderLayout.SOUTH);
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Displays the dialog on the screen
     */
    public void display()
    {
        setVisible(true);
        requestFocus();
    }

    // =========================================================================
    // IMPLEMENTS OBSERVER
    // =========================================================================

    /**
     * Receives notice of look and feel changes
     *
     * @param observable  unused
     * @param arg         unused
     */
    public void update(Observable observable, Object arg)
    {
        updateAppearance();
    }

    // =========================================================================
    // ABSTRACT METHODS
    // =========================================================================

    /**
     * Creates the dialog's control area, where user input is gathered.
     *
     * @return <code>JComponent</code> representing control area
     */
    abstract protected JComponent createControlArea();

    /**
     * Creates the dialog's action area, where user initiates action on input.
     *
     * @return <code>JComponent</code> representing action area
     */
    abstract protected JComponent createActionArea();

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Centers this dialog on its parent
     */
    protected void centerOnParent()
    {
        Container parent = getParent();
        Point topLeft = parent.getLocationOnScreen();

        Dimension parentSize = parent.getSize();
        Dimension size = getSize();

        int x = topLeft.x;
        int y = topLeft.y;

        if (parentSize.width > size.width)
            x += ((parentSize.width - size.width) / 2);

        if (parentSize.height > size.height)
            y += ((parentSize.height - size.height) / 2);

        setLocation(x, y);
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Asks each node in the tree to initialize its UI property with the
     * current Look & Feel
     */
    private void updateAppearance()
    {
        SwingUtilities.updateComponentTreeUI(this);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected JComponent controlArea;
    protected JComponent actionArea;
}

