/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for default ABOUT dialog class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)DefaultAboutDialog.java    1.0 2001/06/07
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;


/**
 * DefaultAboutDialog - Creates a generic ABOUT dialog.
 *
 * @version 1.0 2001/06/07
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
abstract public class DefaultAboutDialog extends AbstractAboutDialog
    implements DefaultAboutDialogKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor specifying title
     *
     * @param title  string specifying dialog title
     */
    public DefaultAboutDialog(String title)
    {
        this((Frame)null, title);
    }

    /**
     * Class constructor specifying owner and title
     *
     * @param owner  parent component
     * @param title  string specifying dialog title
     */
    public DefaultAboutDialog(Frame owner, String title)
    {
        super(owner, title);
        setName("DefaultAboutDialog");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    }

    // =========================================================================
    // PUBLIC INTERFACE
    // =========================================================================

    /**
     * Displays the dialog on the screen, centered on its parent
     */ 
    public void display()
    {
        centerOnParent();
        setVisible(true);
        requestFocus();
    }

    // =========================================================================
    // ABSTRACT METHODS
    // =========================================================================

    /**
     * Creates the banner panel displayed as the main pane
     * in the tabbed pane.
     *
     * @return <code>JPanel</code> representing about panel
     * @see AboutPanel 
     */
    abstract protected JPanel createAboutPanel();

    /**
     * Creates a pane for displaying contact information
     *
     * @return <code>JPanel</code> representing contact panel
     */
    abstract protected JPanel createContactPanel();

    /**
     * Creates a pane for displaying developer credits, et cetera.
     *
     * @return <code>JPanel</code> representing credits panel
     */
    abstract protected JPanel createCreditsPanel();

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Creates the dialog's control area, where user input is gathered.
     *
     * @return <code>JComponent</code> representing control area
     */
    protected JComponent createControlArea()
    {
        return createTabbedPane();
    }

    /**
     * Creates the dialog's action area, where user initiates action on input.
     *
     * @return <code>JComponent</code> representing action area
     */
    protected JComponent createActionArea()
    {
        return createButtonPane();
    }

    /**
     * Implements the dialog action area. This implementation simply
     * adds an OK button which dismisses the dialog when pressed.
     *
     * @return <code>JPanel</code> containing OK button
     */
    protected JComponent createButtonPane()
    {
        JPanel buttonPane = new JPanel();
        String label;
        String toolTipText;

        label = getBundleString(ABOUT_DIALOG_OK_BUTTON_LABEL);
        toolTipText = getBundleString(ABOUT_DIALOG_OK_BUTTON_TOOLTIP);

        JButton okButton = new JButton(label);
        okButton.setToolTipText(toolTipText);
        okButton.setRequestFocusEnabled(true);
        okButton.requestFocus();
        okButton.addActionListener(new OkButtonListener());
        buttonPane.add(okButton);

        return buttonPane;
    }

    /**
     * Creates a System Information panel that provides information
     * about the Java Runtime Environment.
     *
     * @return <code>JPanel</code> containing system information
     * @see org.mdanderson.biostat.util.SysInfoPanel
     */
    protected JPanel createSysInfoPanel()
    {
        return new SysInfoPanel();
    }

    /**
     * Implements the dialog control area.  Current implementation adds
     * several panels to a tabbed pane.
     *
     * @return <code>JTabbedPane</code> containing information panes
     */
    protected JTabbedPane createTabbedPane()
    {
        String aboutTabTitle;
        String creditsTabTitle;
        String contactTabTitle;
        String sysInfoTabTitle;

        aboutTabTitle   = getBundleString(ABOUT_DIALOG_ABOUT_TAB_TITLE);
        creditsTabTitle = getBundleString(ABOUT_DIALOG_CREDITS_TAB_TITLE);
        contactTabTitle = getBundleString(ABOUT_DIALOG_CONTACT_TAB_TITLE);
        sysInfoTabTitle = getBundleString(ABOUT_DIALOG_SYSINFO_TAB_TITLE);

        aboutPanel   = createAboutPanel();
        creditsPanel = createCreditsPanel();
        contactPanel = createContactPanel();
        sysInfoPanel = createSysInfoPanel();

        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.addTab(aboutTabTitle,   aboutPanel);
        tabbedPane.addTab(creditsTabTitle, creditsPanel);
        tabbedPane.addTab(contactTabTitle, contactPanel);
        tabbedPane.addTab(sysInfoTabTitle, sysInfoPanel);
        tabbedPane.setSelectedIndex(0);

        return tabbedPane;
    }

    // =========================================================================
    // INNER CLASSES
    // =========================================================================

    /**
     * OkButtonListener - Responds to action events.
     *
     * @version 0.95 2001/12/07
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class OkButtonListener implements ActionListener
    {
        // =====================================================================
        // IMPLEMENTS ACTIONLISTENER
        // =====================================================================

        /**
         * Invoked when an action occurs.
         * Dismisses the about dialog.
         *
         * @param event  component-specific action event
         */
        public void actionPerformed(ActionEvent event)
        {
            setVisible(false);
            dispose();
        }
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message or null
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;

    static
    {
        final String propertyFile = DefaultAboutDialog.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected JComponent  buttonPane;
    protected JTabbedPane tabbedPane;
    protected JPanel      aboutPanel;
    protected JPanel      creditsPanel;
    protected JPanel      contactPanel;
    protected JPanel      sysInfoPanel;
}

