/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for generic ABOUT panel class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)DefaultAboutPanel.java    1.0 2001/03/22
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.net.URL;
import java.util.Date;
import java.util.ResourceBundle;
import javax.accessibility.AccessibleContext;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;


/**
 * DefaultAboutPanel - Creates a generic ABOUT panel
 *
 * @version 1.0 2001/06/07
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class DefaultAboutPanel extends AbstractAboutPanel
    implements DefaultAboutPanelKeys
{
    // =========================================================================
    // CONSTRUCTORS
    // =========================================================================

    /**
     * Class constructor
     */
    public DefaultAboutPanel()
    {
        super();
        addAboutPanelItems();
        addAccessibilityDescriptions();
    }

    /**
     * Class constructor specifying image buffer
     *
     * @param imageBuffer  array of bytes specifying image buffer
     */
    public DefaultAboutPanel(byte[] imageBuffer)
    {
        this(new ImageIcon(imageBuffer));
    }

    /**
     * Class constructor specifying image file name
     *
     * @param imageFileName  name of file containing image
     */
    public DefaultAboutPanel(String imageFileName)
    {
        this(new ImageIcon(imageFileName));
    }

    /**
     * Class constructor specifying image URL
     *
     * @param imageURL  url of image
     */
    public DefaultAboutPanel(URL imageURL)
    {
        this(new ImageIcon(imageURL));
    }

    /**
     * Class constructor specifying image icon
     *
     * @param imageIcon  image icon
     */
    public DefaultAboutPanel(ImageIcon imageIcon)
    {
        super();
        this.imageIcon = imageIcon;
        addAboutPanelItems();
        addAccessibilityDescriptions();

        final int WIDTH = 299;
        final int HEIGHT = 299;
        setPreferredSize(new Dimension(WIDTH, HEIGHT));
    }

    // =========================================================================
    // PUBLIC INTERFACE (Mutators)
    // =========================================================================

    /**
     * Sets the application name and version to be displayed
     *
     * @param appName     string specifying application name to be displayed
     * @param appVersion  string specifying application version to be displayed
     */
    public void setAppVersionLine(String appName, String appVersion)
    {
        appNameVersionLine.setText(appName + " " + appVersion);
    }

    /**
     * Sets the banner to be displayed
     *
     * @param banner  string specifying banner text to be displayed
     */
    public void setBannerLine(String banner)
    {
/*
        bannerLine.setText(enhanceBanner(banner));
*/
        bannerLine.setText(banner);
    }

    /**
     * Sets the copyright to be displayed
     *
     * @param copyright  string specifying banner text to be displayed
     */
    public void setCopyrightLine(String copyright)
    {
        copyrightLine.setText(copyright);
    }

    /**
     * Shows or hides the copyright information depending on the
     * value of parameter
     *
     * @param visible  if true, shows copyright information; otherwise, hides it
     */
    public void setCopyrightVisible(boolean visible)
    {
        copyrightLine.setVisible(visible);
    }

    /**
     * Sets the legal text to be displayed
     *
     * @param legalities  string specifying legal text to be displayed
     */
    public void setLegalitiesLine(String legalities)
    {
        legalitiesLine.setText(legalities);
    }

    /**
     * Shows or hides the legal information depending on the
     * value of parameter
     *
     * @param visible  if true, shows legal information; otherwise, hides it
     */
    public void setLegalitiesVisible(boolean visible)
    {
        legalitiesLine.setVisible(visible);
    }

    /**
     * Sets the release date to be displayed
     *
     * @param date  <code>Date</code> specifying release date to be displayed
     */
    public void setReleaseDateLine(Date date)
    {
        setReleaseDateLine(date.toString());
    }

    /**
     * Sets the release date to be displayed
     *
     * @param date  string specifying release date to be displayed
     */
    public void setReleaseDateLine(String date)
    {
        releaseDateLine.setText(date);
    }

    // =========================================================================
    // PROTECTED INTERFACE
    // =========================================================================

    /**
     * Adds items to about panel. This implementation adds a banner panel at
     * the top, an info panel at the bottom, and an image panel in the middle.
     */ 
    protected void addAboutPanelItems()
    {
        JPanel bannerPanel = createBannerPanel();
        JPanel imagePanel  = createImagePanel();
        JPanel infoPanel   = createInfoPanel();

        add(bannerPanel, BorderLayout.NORTH);
        add(imagePanel,  BorderLayout.CENTER);
        add(infoPanel,   BorderLayout.SOUTH);
    }

    /**
     * Creates a pane for displaying banner
     *
     * @return <code>JPanel</code> representing banner panel
     */
    protected JPanel createBannerPanel()
    {
        JPanel bannerPanel = new JPanel();
        bannerPanel.setLayout(new BoxLayout(bannerPanel, BoxLayout.Y_AXIS));
        bannerPanel.setBackground(Color.black);

        // Add elements
        {
            Font sansSerif = new Font("SansSerif", Font.BOLD|Font.ITALIC, 36);
            JLabel label = new JLabel("***application name***")
            {
                Dimension dimension = this.getPreferredSize();
                final int WIDTH = 399;
                final int HEIGHT = 50;

                /**
                 * Gets the minimum size of the component
                 *
                 * @return  dimension object indicating this component's
                 *          minimum size
                 */
                public Dimension getMinimumSize()
                {
                    return new Dimension(WIDTH, HEIGHT);
                }

                /**
                 * Gets the maximum size of the component
                 *
                 * @return  dimension object indicating this component's
                 *          maximum size
                 */
                public Dimension getMaximumSize()
                {
                    return new Dimension(WIDTH, HEIGHT);
                }

                /**
                 * Gets the preferred size of the component
                 *
                 * @return  dimension object indicating this component's
                 *          preferred size
                 */
                public Dimension getPreferredSize()
                {
                    return new Dimension(WIDTH, HEIGHT);
                }
            };
            label.setForeground(Color.yellow);
            label.setFont(sansSerif);
            label.setLabelFor(null);
            bannerLine = label;
            bannerPanel.add(bannerLine);
        }

        return bannerPanel;
    }

    /**
     * Creates a pane for displaying image
     *
     * @return <code>JPanel</code> representing image panel
     */
    protected JPanel createImagePanel()
    {
        JPanel imagePanel = new JPanel();
        imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.Y_AXIS));

        // Add elements
        {
            JLabel label = new JLabel(imageIcon);
            // Just in case it's not there
            if (imageIcon == null)
                label.setText("***Missing image***");
            splashLabel = label;
            imagePanel.add(splashLabel);
        }

        return imagePanel;
    }

    /**
     * Creates a pane for displaying application information
     *
     * @return <code>JPanel</code> representing application information panel
     */
    protected JPanel createInfoPanel()
    {
        JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));

        String appNameVersion =
                           getBundleString(ABOUT_PANEL_APP_NAME_VERSION_LABEL);
        String releaseDate = getBundleString(ABOUT_PANEL_RELEASE_DATE_LABEL);
        String copyright = getBundleString(ABOUT_PANEL_COPYRIGHT_LABEL);
        String legalities = getBundleString(ABOUT_PANEL_LEGALITIES_LABEL);

        // Add elements
        {
            JLabel label;
            Font sansSerif;

            sansSerif = new Font("SansSerif", Font.BOLD, 18);
            label = new JLabel(appNameVersion);
            label.setForeground(Color.white);
            label.setFont(sansSerif);
            label.setLabelFor(null);
            appNameVersionLine = label;

            sansSerif = new Font("SansSerif", Font.BOLD, 12);
            label = new JLabel(releaseDate);
            label.setForeground(Color.black);
            label.setFont(sansSerif);
            label.setLabelFor(null);
            releaseDateLine = label;

            label = new JLabel(copyright);
            label.setForeground(Color.black);
            label.setFont(sansSerif);
            label.setLabelFor(null);
            copyrightLine = label;

            label = new JLabel(legalities);
            label.setForeground(Color.black);
            label.setFont(sansSerif);
            label.setLabelFor(null);
            legalitiesLine = label;

            infoPanel.add(appNameVersionLine);
            infoPanel.add(releaseDateLine);
            infoPanel.add(Box.createVerticalStrut(12));
            infoPanel.add(copyrightLine);
            infoPanel.add(legalitiesLine);
        }

        return infoPanel;
    }

    /**
     * Enhances the original banner string.
     *
     * @param banner  string containing banner text 
     * @return        string containing HTML version of banner text
     */
    protected String enhanceBanner(String banner)
    {
        final int DESIRED_HTML_FONT_SIZE = 5;

        String htmlString = convertToMixedSizeUppercase(banner, 5);
        JTextField htmlTextField = new JTextField(20);
        htmlTextField.setText(htmlString);
        System.out.println(htmlTextField.getText());

        return htmlTextField.getText();
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Converts the original input string into an HTML version of itself
     * where lowercase letters are promoted to uppercase letters with a
     * smaller font size.
     *
     * @param origString  string to be converted
     * @param fontSize    HTML font size to use (2..7)
     * @return            string with embedded HTML for fancy display
     * @see #enhanceBanner
     */
    private String convertToMixedSizeUppercase(String origString, int fontSize)
    {
        StringBuffer htmlStringBuffer = new StringBuffer("<html>");

        // Make sure HTML font size is within expected bounds
        {
            final int MIN_HTML_FONT_SIZE = 1;
            final int MAX_HTML_FONT_SIZE = 7;

            if (fontSize < MIN_HTML_FONT_SIZE)
                fontSize = MIN_HTML_FONT_SIZE+1;
            if (fontSize > MAX_HTML_FONT_SIZE)
                fontSize = MAX_HTML_FONT_SIZE;
        }

        htmlStringBuffer.append("<font size=\"" + fontSize + "\">");
        {
            boolean isUpper = false;
            boolean termFont = false;
            for (int i = 0; i < origString.length(); i++)
            {
                isUpper = Character.isUpperCase(origString.charAt(i));

                if (isUpper == true)
                {
                    if (termFont == true)
                    {
                        htmlStringBuffer.append("</font>");
                        termFont = false;
                    }
                    htmlStringBuffer.append(origString.charAt(i));
                }
                else {
                    if (termFont == false)
                    {
                        htmlStringBuffer.append("<font size=\"-1\">");
                        termFont = true;
                    }
                    htmlStringBuffer.append(
                          Character.toUpperCase(origString.charAt(i)));
                }
            }

            if (isUpper == false && termFont == true)
                htmlStringBuffer.append("</font>");
        }
        htmlStringBuffer.append("</font>");

        return htmlStringBuffer.toString();
    }

    /**
     * Adds accessible descriptions to the various components
     */
    private void addAccessibilityDescriptions()
    {
        String propertyKey;
        AccessibleContext ac;

        if (bannerLine != null)
        {
            propertyKey = ABOUT_PANEL_BANNER_DESCRIPTION;
            ac = bannerLine.getAccessibleContext();
            ac.setAccessibleDescription(getBundleString(propertyKey));
        }

        if (appNameVersionLine != null)
        {
            propertyKey = ABOUT_PANEL_APP_NAME_VERSION_DESCRIPTION;
            ac = appNameVersionLine.getAccessibleContext();
            ac.setAccessibleDescription(getBundleString(propertyKey));
        }

        if (releaseDateLine != null)
        {
            propertyKey = ABOUT_PANEL_RELEASE_DATE_DESCRIPTION;
            ac = releaseDateLine.getAccessibleContext();
            ac.setAccessibleDescription(getBundleString(propertyKey));
        }

        if (copyrightLine != null)
        {
            propertyKey = ABOUT_PANEL_COPYRIGHT_DESCRIPTION;
            ac = copyrightLine.getAccessibleContext();
            ac.setAccessibleDescription(getBundleString(propertyKey));
        }

        if (legalitiesLine != null)
        {
            propertyKey = ABOUT_PANEL_LEGALITIES_DESCRIPTION;
            ac = legalitiesLine.getAccessibleContext();
            ac.setAccessibleDescription(getBundleString(propertyKey));
        }
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static ResourceBundle bundle;

    static
    {
        final String propertyFile = DefaultAboutPanel.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    protected JLabel bannerLine;
    protected JLabel appNameVersionLine;
    protected JLabel copyrightLine;
    protected JLabel legalitiesLine;
    protected JLabel releaseDateLine;
    protected JLabel splashLabel;
    protected ImageIcon imageIcon;
}

