/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for GUI utility routines class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)GuiUtil.java    1.0 2002/01/01
 *
 * Copyright (c) 2002 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Container;
import java.awt.event.ActionListener;
import javax.accessibility.AccessibleContext;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.border.TitledBorder;


/**
 * GuiUtil - Implements GUI utility routines.
 *
 * @version 1.0 2002/01/01
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
public class GuiUtil
{
    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Adds a titled border to a container
     *
     * @param component    object to which to apply the border
     * @param borderTitle  string specifying text to be displayed on border
     */
    static public void addTitledBorder(JComponent component, String borderTitle)
    {
        TitledBorder titledBorder;

        titledBorder = BorderFactory.createTitledBorder(borderTitle);
        component.setBorder(titledBorder);
    }

    /**
     * Creates a generic menu item
     *
     * @param label           string specifying label text
     * @param mnemonic        string specifying keyboard mnemonic
     * @param description     string specifying accessible description
     * @param toolTipText     string specifying tool tip text to be displayed
     * @param actionListener  <code>ActionListener</code> to be added to the
     *                        menuitem
     * @return                instance of <code>JMenuItem</code> object
     */
    static public JMenuItem createMenuItem(String label,
                                           String mnemonic,
                                           String description,
                                           String toolTipText,
                                   ActionListener actionListener)
    {
        JMenuItem menuItem = new JMenuItem(label);
        menuItem.setMnemonic(mnemonic.charAt(0));
        AccessibleContext ac = menuItem.getAccessibleContext();
        ac.setAccessibleDescription(description);
        menuItem.setToolTipText(toolTipText);
        menuItem.addActionListener(actionListener);
        if (actionListener == null)
            menuItem.setEnabled(false);

        return menuItem;
    }

    /**
     * Creates a radiobutton menu item
     *
     * @param label           string specifying label text
     * @param mnemonic        string specifying keyboard mnemonic
     * @param description     string specifying accessible description
     * @param toolTipText     string specifying tool tip text to be displayed
     * @param actionListener  <code>ActionListener</code> to be added to the
     *                        menuitem
     * @return                instance of <code>JRadioButtonMenuItem</code>
     *                        object
     */
    static public JMenuItem createRadioButtonMenuItem(String label,
                                                      String mnemonic,
                                                      String description,
                                                      String toolTipText,
                                              ActionListener actionListener)
    {
        JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem(label);
        menuItem.setMnemonic(mnemonic.charAt(0));
        AccessibleContext ac = menuItem.getAccessibleContext();
        ac.setAccessibleDescription(description);
        menuItem.setToolTipText(toolTipText);
        menuItem.addActionListener(actionListener);
        if (actionListener == null)
            menuItem.setEnabled(false);

        return menuItem;
    }

    /**
     * Displays an error message dialog
     *
     * @param parent   <code>Container</code> object
     * @param title    string specifying dialog title
     * @param message  string specifying error message to be displayed
     */
    static public void displayErrorDialog(Container parent,
                                          String title,
                                          String message)
    {
        if (parent instanceof JDesktopPane)
            JOptionPane.showInternalMessageDialog(parent,
                                                  message,
                                                  title,
                                                  JOptionPane.ERROR_MESSAGE);
        else
            JOptionPane.showMessageDialog(parent,
                                          message,
                                          title,
                                          JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Displays an information message dialog
     *
     * @param parent   <code>Container</code> object
     * @param title    string specifying dialog title
     * @param message  string specifying information message to be displayed
     */
    static public void displayInfoDialog(Container parent,
                                         String title,
                                         String message)
    {
        if (parent instanceof JDesktopPane)
            JOptionPane.showInternalMessageDialog(parent,
                                                  message,
                                                  title,
                                      JOptionPane.INFORMATION_MESSAGE);
        else
            JOptionPane.showMessageDialog(parent,
                                          message,
                                          title,
                              JOptionPane.INFORMATION_MESSAGE);
    }

    /*
     * Displays a notice message dialog
     *
     * @param parent   <code>Container</code> object
     * @param title    string specifying dialog title
     * @param message  string specifying information message to be displayed
     * @param delay    milliseconds to display the dialog
     * @see ExtendedOptionPane
     */
    static public void displayNoticeDialog(Container parent,
                                           String title,
                                           String message,
                                           long delay)
    {
        if (parent instanceof JDesktopPane)
            ExtendedOptionPane.showInternalNoticeDialog(parent,
                                                        title,
                                                        message,
                                                        delay);
        else
            ExtendedOptionPane.showNoticeDialog(parent,
                                                title,
                                                message,
                                                delay);
    }
}

