/*=================== FILE ABSTRACT ==========================================
 *
 * Synopsis::         Source for system information panel class
 *
 * File Name::        %M%
 * Version::          %I%
 * Type::             source
 * Date Completed::   %G%
 *
 * Environment::
 *     Language::     Java
 *     Computer::     Compaq/Alpha
 *     OS::           DigitalUNIX
 *     OSVersion::    4.0
 *
 * Module Information::
 *     Owner::        MDACC
 *     Department::   Biostatistics
 *     Researcher::   N/A
 *     Application::  <generic>
 *
 *============================================================================*/

/*
 * @(#)SysInfoPanel.java    1.0 2001/06/07
 *
 * Copyright (c) 2001 M.D. Anderson Cancer Center
 * 1515 Holcombe, Houston, TX, 77030, U.S.A.
 * All rights reserved.
 */

package org.mdanderson.biostat.util;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Timer;


/**
 * SysInfoPanel - Creates a generic system information panel.
 *
 * @version 1.0 2001/06/07
 * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
 */
class SysInfoPanel extends JPanel
    implements SysInfoPanelKeys
{
    // =========================================================================
    // CONSTRUCTOR
    // =========================================================================

    /**
     * Class constructor
     */
    public SysInfoPanel()
    {
        super();
        setName("SysInfoPanel");

        addSysInfoPanelItems();
    }

    // =========================================================================
    // PRIVATE INTERFACE
    // =========================================================================

    /**
     * Adds fields to display the system information
     */
    private void addSysInfoPanelItems()
    {
        final int ROWS    = -1;
        final int COLUMNS = 2;
        final int HGAP = 5;
        final int VGAP = 5;

        setLayout(new GridLayout(ROWS, COLUMNS, HGAP, VGAP));

        int numItemsToDisplay = sysInfoPropPairs.length+2;
        String propLabel[] = new String[numItemsToDisplay];
        String propValue[] = new String[numItemsToDisplay];

        // Get system properties
        try
        {
            for (int i = 0; i < sysInfoPropPairs.length; i++)
            {
                String propKey = sysInfoPropPairs[i][1];
                propLabel[i] = getBundleString(sysInfoPropPairs[i][0]);
                try
                {
                    propValue[i] = System.getProperty(propKey);
                }
                catch (SecurityException se)
                {
                    System.err.println(se);
                    propValue[i] = null;
                }
            }
        }
        catch (Exception e)
        {
            System.err.println(e);
        }

        int totalMemIndex;
        int freeMemIndex;

        // Get runtime information
        {
            totalMemory = new Memory(Runtime.getRuntime().totalMemory() >> 10);
            freeMemory = new Memory(Runtime.getRuntime().freeMemory() >> 10);
            int i = sysInfoPropPairs.length;

            propLabel[i] = getBundleString(SYSINFO_TOTAL_VM_MEMORY_LABEL);
            propValue[i] = totalMemory.toString();
            totalMemIndex = i++;

            propLabel[i] = getBundleString(SYSINFO_FREE_VM_MEMORY_LABEL);
            propValue[i] = freeMemory.toString();
            freeMemIndex = i;
        }

        // Put the information on the panel
        {
            JLabel label;
            JLabel value = null;
            String notAvailable = getBundleString(SYSINFO_MISSING_VALUE_TEXT);
            Font labelFont = new Font("Monospaced", Font.BOLD, 10);
            Font valueFont = new Font("Monospaced", Font.PLAIN, 10);

            for (int i = 0; i < numItemsToDisplay; i++)
            {
                if (propValue[i] == null || propValue[i].equals(""))
                    propValue[i] = notAvailable;

                // Use update capable label for memory (last two)
                if (i == totalMemIndex)
                    value = new TotalMemoryLabel(totalMemory);
                else if (i == freeMemIndex)
                    value = new FreeMemoryLabel(freeMemory);
                else
                    value = new JLabel(propValue[i]);
                value.setForeground(Color.black);
                value.setFont(valueFont);
                value.setLabelFor(null);

                label = new JLabel(propLabel[i]);
                label.setForeground(Color.black);
                label.setFont(labelFont);
                label.setLabelFor(value);

                add(label);
                add(value);
            }
        }
    }

    // =========================================================================
    // INNER CLASSES
    // =========================================================================

    /**
     * Memory - Used to represent the system memory (RAM) 
     *
     * @version 1.0 2001/03/22
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class Memory
    {
        // =====================================================================
        // CONSTRUCTORS
        // =====================================================================

        /**
         * Class constructor
         */
        public Memory()
        {
            this(0);
        }

        /**
         * Class constructor specifying amount of memory in kilobytes
         *
         * @param amount  amount of memory in kilobytes
         * @throws IllegalArgumentException  if amount is negative
         */
        public Memory(long amount)
        {
            setAmount(amount);
            unit = getBundleString(SYSINFO_KILOBYTE_TEXT);
        }

        // =====================================================================
        // PUBLIC INTERFACE
        // =====================================================================

        /**
         * Returns a string representation of the object.
         *
         * @return string representation of the object
         */
        public String toString()
        {
             return Long.toString(amount) + " " + unit;
        }

        /**
         * Returns the amount of memory
         *
         * @return  amount of memory in kilobytes
         */
        public long getAmount()
        {
             return amount;
        }

        /**
         * Sets the amount of memory
         *
         * @param  amount of memory in kilobytes
         * @throws IllegalArgumentException  if amount is negative
         */
        public void setAmount(long amount)
        {
             if (amount < 0)
             {
                 String detail = getBundleString(ERR_NEGATIVE_MEMORY_AMOUNT);
                 throw new IllegalArgumentException(detail);
             }

             this.amount = amount;
        }

        // =====================================================================
        // OBJECT ATTRIBUTES
        // =====================================================================

        private String unit;
        private long amount;
    }

    /**
     * MemoryLabel - Enhanced label that dynamically tracks system memory.
     *
     * @version 1.0 2001/06/07
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    abstract class MemoryLabel extends JLabel
    {
        // =====================================================================
        // CONSTRUCTOR
        // =====================================================================

        /**
         * Class constructor specifying memory
         *
         * @param m  reference to <code>Memory</code> object
         */
        public MemoryLabel(Memory m)
        {
            super();
            this.memory = m;

            int delay = SECONDS_TO_DELAY * ONE_SECOND;
            Timer timer = new Timer(delay, new ActionListener()
            {
                /**
                 * Invoked when an action occurs.
                 * Updates the memory amount and updates the label accordingly
                 *
                 * @param event  component-specific action event
                 */
                public void actionPerformed(ActionEvent event)
                {
                    memory.setAmount(getRuntimeMemory());
                    setText(memory.toString());
                }
            });
            timer.setInitialDelay(0);
            timer.start();
        }

        // =====================================================================
        // PROTECTED INTERFACE
        // =====================================================================

        /**
         * Returns value with which to update memory object's amount of memory.
         *
         * @return  new value for amount of memory in kilobytes
         */
        abstract protected long getRuntimeMemory();

        // =====================================================================
        // CONSTANTS
        // =====================================================================

        static final int ONE_SECOND = 1000;
        static final int SECONDS_TO_DELAY = 5;

        // =====================================================================
        // OBJECT ATTRIBUTES
        // =====================================================================

        protected Memory memory;
    }

    /**
     * TotalMemoryLabel - Enhanced label that dynamically tracks system memory.
     * Tracks the total amount of memory in the JVM.
     *
     * @version 1.0 2001/06/07
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class TotalMemoryLabel extends MemoryLabel
    {
        // =====================================================================
        // CONSTRUCTOR
        // =====================================================================

        /**
         * Class constructor specifying memory
         *
         * @param totalMemory  reference to <code>Memory</code> object
         */
        public TotalMemoryLabel(Memory totalMemory)
        {
            super(totalMemory);
        }

        // =====================================================================
        // PROTECTED INTERFACE
        // =====================================================================

        /**
         * Returns the current amount of JVM total memory in kilobytes
         *
         * @return  new value for amount of memory in kilobytes
         * @see Runtime#totalMemory
         */
        protected long getRuntimeMemory()
        {
            return Runtime.getRuntime().totalMemory() >> 10;
        }
    }

    /**
     * FreeMemoryLabel - Enhanced label that dynamically tracks system memory.
     * Tracks the amount of free memory in the JVM.
     *
     * @version 1.0 2001/06/07
     * @author <a href="mailto:roebuck@mdanderson.org">P. Roebuck</a>
     */
    class FreeMemoryLabel extends MemoryLabel
    {
        // =====================================================================
        // CONSTRUCTOR
        // =====================================================================

        /**
         * Class constructor specifying memory
         *
         * @param freeMemory  reference to <code>Memory</code> object
         */
        public FreeMemoryLabel(Memory freeMemory)
        {
            super(freeMemory);
        }

        // =====================================================================
        // PROTECTED INTERFACE
        // =====================================================================

        /**
         * Returns the current amount of JVM free memory in kilobytes
         *
         * @return  new value for amount of memory in kilobytes
         * @see Runtime#freeMemory
         */
        protected long getRuntimeMemory()
        {
            return Runtime.getRuntime().freeMemory() >> 10;
        }
    }

    // =========================================================================
    // MAIN
    // =========================================================================

    /**
     * Main entry-point for the SysInfoPanel class
     *
     * @param argv  string array containing command line arguments
     */
    public static void main(String[] argv)
    {
        JFrame frame = new JFrame("SysInfoPanelDemo");
        frame.getContentPane().add(new SysInfoPanel());
        frame.addWindowListener(new WindowAdapter()
        {
            /**
             * Invoked when a window is in the process of being closed.
             * Terminates the JVM.
             *
             * @param event  low-level window status event
             */
            public void windowClosing(WindowEvent event)
            {
                System.exit(0);
            }
        });
        frame.setSize(425, 200);
        frame.setVisible(true);
    }

    // =========================================================================
    // CLASS INTERFACE
    // =========================================================================

    /**
     * Fetches messages from resource bundle.
     *
     * @param propertyKey  string containing property key for format string
     * @return             string containing message or null
     */
    static String getBundleString(String propertyKey)
    {
        return (bundle == null) ? null : bundle.getString(propertyKey);
    }

    // =========================================================================
    // CLASS ATTRIBUTES
    // =========================================================================

    private static final String keyOsName           = "os.name";
    private static final String keyOsVersion        = "os.version";
    private static final String keyArchitecture     = "os.arch";
    private static final String keyJavaVersion      = "java.version";
    private static final String keyJvmName          = "java.vm.name";
    private static final String keyJvmVersion       = "java.vm.version";
    private static final String keyJvmInfo          = "java.vm.info";
    private static final String keyJitCompiler      = "java.compiler";
    private static final String keyJavaClassVersion = "java.class.version";

    private static final String sysInfoPropPairs [][] = {
        { SYSINFO_OS_LABEL,                 keyOsName           },
        { SYSINFO_OS_VERSION_LABEL,         keyOsVersion        },
        { SYSINFO_ARCHITECTURE_LABEL,       keyArchitecture     },
        { SYSINFO_JAVA_VERSION_LABEL,       keyJavaVersion      },
        { SYSINFO_JVM_NAME_LABEL,           keyJvmName          },
        { SYSINFO_JVM_VERSION_LABEL,        keyJvmVersion       },
        { SYSINFO_JVM_INFO_LABEL,           keyJvmInfo          },
        { SYSINFO_JIT_COMPILER_LABEL,       keyJitCompiler      },
        { SYSINFO_JAVA_CLASS_VERSION_LABEL, keyJavaClassVersion }
    };

    private static ResourceBundle bundle;

    static
    {
        final String propertyFile = SysInfoPanel.class.getName();
        bundle = ResourceBundle.getBundle(propertyFile);
    }

    // =========================================================================
    // OBJECT ATTRIBUTES
    // =========================================================================

    private Memory totalMemory;
    private Memory freeMemory;
}

